#include <string.h>
#include "MEM.h"
#include "DBG.h"
#include "dvm_pri.h"

static void
check_array(DVM_ObjectRef array, int index,
            DVM_Executable *exe, Function *func, int pc)
{
    if (array.data == NULL) {
        dvm_error(exe, func, pc, NULL_POINTER_ERR,
                  MESSAGE_ARGUMENT_END);
    }
    if (index < 0 || index >= array.data->u.array.size) {
        dvm_error(exe, func, pc, INDEX_OUT_OF_BOUNDS_ERR,
                  INT_MESSAGE_ARGUMENT, "index", index,
                  INT_MESSAGE_ARGUMENT, "size", array.data->u.array.size,
                  MESSAGE_ARGUMENT_END);
    }
}

int
DVM_array_get_int(DVM_VirtualMachine *dvm, DVM_ObjectRef array, int index)
{
    check_array(array, index,
                dvm->current_executable->executable,
                dvm->current_function, dvm->pc);

    return array.data->u.array.u.int_array[index];
}

double
DVM_array_get_double(DVM_VirtualMachine *dvm, DVM_ObjectRef array, int index)
{
    check_array(array, index,
                dvm->current_executable->executable,
                dvm->current_function, dvm->pc);

    return array.data->u.array.u.double_array[index];
}

DVM_ObjectRef
DVM_array_get_object(DVM_VirtualMachine *dvm, DVM_ObjectRef array, int index)
{
    check_array(array, index,
                dvm->current_executable->executable,
                dvm->current_function, dvm->pc);

    return array.data->u.array.u.object[index];
}

void
DVM_array_set_int(DVM_VirtualMachine *dvm, DVM_ObjectRef array, int index,
                  int value)
{
    check_array(array, index,
                dvm->current_executable->executable,
                dvm->current_function, dvm->pc);

    array.data->u.array.u.int_array[index] = value;
}

void
DVM_array_set_double(DVM_VirtualMachine *dvm, DVM_ObjectRef array, int index,
                     double value)
{
    check_array(array, index,
                dvm->current_executable->executable,
                dvm->current_function, dvm->pc);

    array.data->u.array.u.double_array[index] = value;
}

void
DVM_array_set_object(DVM_VirtualMachine *dvm, DVM_ObjectRef array, int index,
                     DVM_ObjectRef value)
{
    check_array(array, index,
                dvm->current_executable->executable,
                dvm->current_function, dvm->pc);

    array.data->u.array.u.object[index] = value;
}
